@@ -27,9 +27,7 @@ ssm_ulg::ssm_ulg(const Rcpp::List model,
     theta(Rcpp::as<arma::vec>(model["theta"])), 
     engine(seed), zero_tol(zero_tol),
     HH(arma::vec(Htv * (n - 1) + 1)), RR(arma::cube(m, m, Rtv * (n - 1) + 1)),
-    xbeta(arma::vec(n, arma::fill::zeros)), 
-    update_fn(Rcpp::as<Rcpp::Function>(model["update_fn"])), 
-    prior_fn(Rcpp::as<Rcpp::Function>(model["prior_fn"])) {
+    xbeta(arma::vec(n, arma::fill::zeros)) {
   
   if(xreg.n_cols > 0) {
     compute_xbeta();
@@ -46,7 +44,6 @@ ssm_ulg::ssm_ulg(
   const arma::vec& D, const arma::mat& C, 
   const arma::mat& xreg, const arma::vec& beta,
   const arma::vec& theta, const unsigned int seed,
-  const Rcpp::Function update_fn, const Rcpp::Function prior_fn,
   const double zero_tol) :
   y(y), Z(Z), H(H), T(T), R(R), a1(a1), P1(P1), D(D), C(C), 
   xreg(xreg), beta(beta), n(y.n_elem), m(a1.n_elem), k(R.n_cols),
@@ -54,8 +51,7 @@ ssm_ulg::ssm_ulg(
   Dtv(D.n_elem > 1), Ctv(C.n_cols > 1),
   theta(theta), engine(seed), zero_tol(zero_tol), 
   HH(arma::vec(Htv * (n - 1) + 1)), RR(arma::cube(m, m, Rtv * (n - 1) + 1)),
-  xbeta(arma::vec(n, arma::fill::zeros)), update_fn(update_fn),
-  prior_fn(prior_fn){
+  xbeta(arma::vec(n, arma::fill::zeros)) {
   
   if(xreg.n_cols > 0) {
     compute_xbeta();
@@ -65,7 +61,7 @@ ssm_ulg::ssm_ulg(
 }
 
 
-void ssm_ulg::update_model(const arma::vec& new_theta) {
+void ssm_ulg::update_model(const arma::vec& new_theta, const Rcpp::Function update_fn) {
   
   Rcpp::List model_list =
     update_fn(Rcpp::NumericVector(new_theta.begin(), new_theta.end()));
@@ -103,7 +99,7 @@ void ssm_ulg::update_model(const arma::vec& new_theta) {
   theta = new_theta;
 }
 
-double ssm_ulg::log_prior_pdf(const arma::vec& x) const {
+double ssm_ulg::log_prior_pdf(const arma::vec& x, const Rcpp::Function prior_fn) const {
   return Rcpp::as<double>(prior_fn(Rcpp::NumericVector(x.begin(), x.end())));
 }
 
@@ -677,7 +673,7 @@ void ssm_ulg::psi_filter(const unsigned int nsim, arma::cube& alpha) {
 
 
 arma::cube ssm_ulg::predict_sample(const arma::mat& theta_posterior,
-  const arma::mat& alpha, const unsigned int predict_type) {
+  const arma::mat& alpha, const unsigned int predict_type, const Rcpp::Function update_fn) {
   
   unsigned int d = 1;
   if (predict_type == 3) d = m;
@@ -686,7 +682,7 @@ arma::cube ssm_ulg::predict_sample(const arma::mat& theta_posterior,
   arma::cube sample(d, n, n_samples);
   
   for (unsigned int i = 0; i < n_samples; i++) {
-    update_model(theta_posterior.col(i));
+    update_model(theta_posterior.col(i), update_fn);
     a1 = alpha.col(i);
     sample.slice(i) = sample_model(predict_type);
   }
@@ -729,14 +725,14 @@ arma::mat ssm_ulg::sample_model(const unsigned int predict_type) {
 
 
 arma::cube ssm_ulg::predict_past(const arma::mat& theta_posterior,
-  const arma::cube& alpha, const unsigned int predict_type) {
+  const arma::cube& alpha, const unsigned int predict_type, const Rcpp::Function update_fn) {
   
   unsigned int n_samples = theta_posterior.n_cols;
   arma::cube samples(p, n, n_samples);
   
   std::normal_distribution<> normal(0.0, 1.0);
   for (unsigned int i = 0; i < n_samples; i++) {
-    update_model(theta_posterior.col(i));
+    update_model(theta_posterior.col(i), update_fn);
     for (unsigned int t = 0; t < n; t++) {
       samples.slice(i).col(t) =  xbeta(t) + D(t * Dtv) +
         arma::as_scalar(Z.col(t * Ztv).t() * alpha.slice(i).col(t));
